<?php

class RCPayment {

	public function __construct ($type) {

		$this->type = $type;
		$this->error = '';
		$this->authCode = RCUtilities::loadSessionData('RCPayment::authCode', '');

		switch ($this->type) {

			case 'stripe':

				\Stripe\Stripe::setApiKey(trim(RCUtilities::$config->stripe->APIKey));

			break;

			case 'mollie':

				$this->mollie = new Mollie_API_Client();

				try {

					$this->mollie->setApiKey(trim(RCUtilities::$config->mollie->APIKey));

				}

				catch (Mollie_API_Exception $e) {

					$this->error = 'API call failed: ' . htmlspecialchars($e->getMessage());

				}

			break;

			case 'paypal':

				$this->rest = curl_init();

				curl_setopt($this->rest, CURLOPT_CAINFO, dirname(__FILE__) . '/../helpers/PayPal/cacert.pem');
				curl_setopt($this->rest, CURLOPT_FAILONERROR, TRUE);
				curl_setopt($this->rest, CURLOPT_RETURNTRANSFER, TRUE);
				curl_setopt($this->rest, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);

				if (defined('CURL_SSLVERSION_TLSv1_2')) {

					curl_setopt($this->rest, CURLOPT_SSLVERSION, CURL_SSLVERSION_TLSv1_2);

				}

			break;

		}

	}

	public function check () {

		switch ($this->type) {

			case 'paypal':

				return $this->authorizePayPal();

			break;

			case 'stripe':

				return $this->checkStripe();

			break;

			case 'mollie':

				return empty($this->error);

			break;

		}

	}

	public function authorize ($options = NULL) {

		switch ($this->type) {

			case 'paypal':

				return $this->authorizePayPal();

			break;

			case 'stripe':

			// 	return $this->authorizeStripe($options->customer, $options->cardHolderFirstName . ' ' . $options->cardHolderLastName, $options->cardNumber, $options->cardExpMonth, $options->cardExpYear, $options->cardCVC);

			// break;

			case 'order':
			case 'cash':
			case 'mollie':

				RCUtilities::saveSessionData('RCPayment::authCode', TRUE);

				return TRUE;

			break;

		}

		return FALSE;

	}

	public function prepare ($cart) {

		switch ($this->type) {

			case 'paypal':

				return $this->preparePayPal($cart);

			break;

		}

		return TRUE;

	}

	public function execute ($options = NULL) {

		if (empty($this->authCode)) {

			$this->error = 'Missing payment auth code.';

			return FALSE;

		}

		switch ($this->type) {

			case 'paypal':

				return $this->executePayPal($options->paymentID, $options->payerID);

			break;

			case 'stripe':

				return $this->executeStripe($options->cart);

			break;

			case 'mollie':

				return $this->executeMollie($options->cart);

			break;

			case 'order':
			case 'cash':

				return '';

			break;

		}

		return FALSE;

	}

	private function checkStripe () {

		if (RCUtilities::$config->stripe->APIKey == '') {

			$this->error = 'Missing Stripe API Key.';

			return FALSE;

		}

		try {

			\Stripe\Balance::retrieve();

		}

		catch (Exception $e) {

			$body = $e->getJsonBody();
			$error  = $body['error'];

			$this->error = $error['message'];

			return FALSE;

		}

	}

	private function authorizePayPal () {

		curl_setopt($this->rest, CURLOPT_URL, 'https://api.' . (RCUtilities::$settings->payments->paypal->sandbox ? 'sandbox.' : '') . 'paypal.com/v1/oauth2/token');
		curl_setopt($this->rest, CURLOPT_USERPWD, trim(RCUtilities::$config->paypal->clientID) . ':' . trim(RCUtilities::$config->paypal->secret));
		curl_setopt($this->rest, CURLOPT_POSTFIELDS, 'grant_type=client_credentials');

		$result = json_decode(curl_exec($this->rest));

		if (isset($result->access_token)) {

			$this->authCode = RCUtilities::saveSessionData('RCPayment::authCode', $result->access_token);

			return TRUE;

		}

		else {

			$this->error = curl_error($this->rest);

			return FALSE;

		}

	}

	private function authorizeStripe ($customer, $cardHolder, $cardNumber, $cardExpMonth, $cardExpYear, $cardCVC) {

		if (empty(RCUtilities::$config->stripe->APIKey)) {

			$this->error = 'Missing Stripe API Key.';

			return FALSE;

		}

		try {

			$response = \Stripe\Token::create(array(

				'card' => array(

					'name'            => $cardHolder,
					'number'          => $cardNumber,
					'exp_month'       => $cardExpMonth,
					'exp_year'        => $cardExpYear,
					'cvc'             => $cardCVC,
					'address_line1'   => $customer->name,
					'address_line2'   => $customer->street,
					'address_city'    => $customer->city,
					'address_zip'     => $customer->ZIPCode,
					'address_state'   => $customer->state,
					'address_country' => $customer->country

				)

			));

			$this->authCode = RCUtilities::saveSessionData('RCPayment::authCode', $response->id);

			return TRUE;

		}

		catch (Exception $e) {

			$body = $e->getJsonBody();
			$error  = $body['error'];

			$this->error = ($error['type'] == 'card_error') ? RCUtilities::$localization->cart->creditCardError : $error['message'];

			return FALSE;

		}

	}

	private function preparePayPal ($cart) {

		$data = (object) array();

		$data->intent = 'sale';
		$data->redirect_urls = (object) array(

			'return_url' => RCUtilities::getURL(array(

				0 => 'cart',
				1 => 'review',
				2 => ''

			), TRUE),
			'cancel_url' =>	RCUtilities::getURL(array(

				0 => 'cart',
				1 => 'payment',
				2 => ''

			), TRUE)

		);
		$data->payer = (object) array(

			'payment_method' => 'paypal'

		);

		$data->transactions = array();
		$data->transactions[] = (object) array(

			'amount'        => (object) array(

				'currency' => RCUtilities::$localization->currency->code,
				'total'    => number_format($cart->total, 2, '.', ''),
				'details'  => (object) array(

					'subtotal' => number_format(number_format($cart->total, 2, '.', '') - number_format($cart->taxes, 2, '.', ''), 2, '.', ''),
					'tax'      => number_format($cart->taxes, 2)

				)

			),
			'description'   => str_replace('%ORDER_ID%', $cart->ID, RCUtilities::$localization->order->title),
			'item_list'     => (object) array(

				'items' => array(

					(object) array(

						'quantity' => 1,
						'name'     => str_replace('%ORDER_ID%', $cart->ID, RCUtilities::$localization->order->title),
						'price'    => number_format(number_format($cart->total, 2, '.', '') - number_format($cart->taxes, 2, '.', ''), 2, '.', ''),
						'sku'      => $cart->ID,
						'currency' => RCUtilities::$localization->currency->code

					)

				)

			)

		);

		curl_setopt($this->rest, CURLOPT_URL, 'https://api.' . (RCUtilities::$settings->payments->paypal->sandbox ? 'sandbox.' : '') . 'paypal.com/v1/payments/payment');
		curl_setopt($this->rest, CURLOPT_POSTFIELDS, json_encode($data));
		curl_setopt($this->rest, CURLOPT_HTTPHEADER, array(

			'Content-Type: application/json',
			'Authorization: Bearer ' . $this->authCode

		));

		$result = json_decode(curl_exec($this->rest));

		if (isset($result->links)) {

			foreach ($result->links as $link) {

				if ($link->rel == 'approval_url') {

					return $link->href;

				}

			}

		}

		$this->error = curl_error($this->rest);

		return FALSE;

	}

	private function executePayPal ($paymentID, $payerID) {

		$data = (object) array(

			'payer_id' => $payerID

		);

		curl_setopt($this->rest, CURLOPT_URL, 'https://api.' . (RCUtilities::$settings->payments->paypal->sandbox ? 'sandbox.' : '') . 'paypal.com/v1/payments/payment/' . $paymentID . '/execute/');
		curl_setopt($this->rest, CURLOPT_POSTFIELDS, json_encode($data));
		curl_setopt($this->rest, CURLOPT_HTTPHEADER, array(

			'Content-Type: application/json',
			'Authorization: Bearer ' . $this->authCode

		));

		$payment = json_decode(curl_exec($this->rest));

		if (isset($payment->state) && $payment->state == 'approved') {

			return $payment->transactions[0]->related_resources[0]->sale->id;

		}

		else {

			$this->error = 'Error processing PayPal payment.' . PHP_EOL . PHP_EOL . curl_error($this->rest);

			return FALSE;

		}

	}

	public function executeStripe ($cart) {

		try {

			$total = round($cart->total * ((RCUtilities::$localization->currency->code == 'JPY') ? 1 : 100));

			if (strpos(RCUtilities::$config->stripe->APIKey, 'sk_test_') === 0) {

				\Stripe\Stripe::setVerifySslCerts(false);

			}

			$description = '';

			foreach ($cart->items as $item) {

				$description .= $item->quantity . ' × ' . $item->product->name;
				$description .= ' (';
				$description .= 'Version: ' . $item->product->version;

				if (!empty($item->product->options)) {

					$description .= ' - Options: ' . implode(', ' , $item->product->options);

				}

				$description .= ') ';

			}

			$session = \Stripe\Checkout\Session::create([

				'customer_email'       => $cart->customer->emailAddress,
				'payment_method_types' => ['card'],
				'success_url'          => RCUtilities::getURL(array(

					0 => 'cart',
					1 => 'checkout',
					2 => 'complete'

				), TRUE),
				'cancel_url'           => RCUtilities::getURL(array(

					0 => 'cart',
					1 => 'review'

				), TRUE),
				'line_items'           => [[

					'name'        => 'Order ' . $cart->ID,
					'description' => trim(strip_tags($description)),
					'amount'      => $total,
					'currency'    => RCUtilities::$localization->currency->code,
					'quantity'    => 1

				]]

			]);

			return $session->id;

			// $response = \Stripe\Charge::create(array(

			// 	'amount'        => $total,
			// 	'currency'      => RCUtilities::$localization->currency->code,
			// 	'source'        => $this->authCode,
			// 	'description'   => str_replace('%ORDER_ID%', $cart->ID, RCUtilities::$localization->order->title),
			// 	'receipt_email' => $cart->customer->emailAddress

			// ));

			// return $response->id;

		}

		catch (Exception $e) {

			$body = $e->getJsonBody();
			$error  = $body['error'];

			$this->error = $error['message'];

			return FALSE;

		}

	}

	private function executeMollie ($cart) {

		try {

			$payments = $this->mollie->payments->all();

			foreach ($payments as $payment) {

				if ($payment->status == 'paid' && (isset($payment->metadata->order_id) && $payment->metadata->order_id == $cart->ID)) {

					return $payment->id;

				}

			}

			try {

				$payment = $this->mollie->payments->create(array(

					'amount'      => $cart->total,
					'description' => str_replace('%ORDER_ID%', $cart->ID, RCUtilities::$localization->order->title),
					'redirectUrl' => RCUtilities::getURL(array(

						0 => 'cart',
						1 => 'checkout'

					), TRUE),
					'metadata'    => array(

						'order_id' => $cart->ID

					)

				));

				RCUtilities::redirectToURL($payment->getPaymentUrl());

			}

			catch (Mollie_API_Exception $e) {

				$this->error = 'API call failed: ' . htmlspecialchars($e->getMessage()) . ' on field ' . htmlspecialchars($e->getField());

				return FALSE;

			}

		}

		catch (Mollie_API_Exception $e) {

			$this->error = 'API call failed: ' . htmlspecialchars($e->getMessage());

			return FALSE;

		}

	}

}

?>